@@ -190,6 +190,14 @@ val wakeup_exn : 'a u -> exn -> unit
   (** [wakeup_exn t e] makes the sleeping thread [t] fail with the
       exception [e]. *)
 
+val wakeup_later : 'a u -> 'a -> unit
+  (** Same as {!wakeup} but it is not guaranteed that the thread will
+      be wakeup immediately. *)
+
+val wakeup_later_exn : 'a u -> exn -> unit
+  (** Same as {!wakeup_exn} but it is not guaranteed that the thread
+      will be wakeup immediately. *)
+
 val waiter_of_wakener : 'a u -> 'a t
   (** Returns the thread associated to a wakener. *)
 
@@ -226,7 +234,7 @@ val on_cancel : 'a t -> (unit -> unit) -> unit
 val cancel : 'a t -> unit
   (** [cancel t] cancels the threads [t]. This means that the deepest
       sleeping thread created with [task] and connected to [t] is
-      wakeup with the exception {!Canceled}.
+      woken up with the exception {!Canceled}.
 
       For example, in the following code:
 
@@ -235,7 +243,7 @@ val cancel : 'a t -> unit
         cancel (waiter >> printl "plop")
       ]}
 
-      [waiter] will be waked up with {!Canceled}.
+      [waiter] will be woken up with {!Canceled}.
   *)
 
 val pick : 'a t list -> 'a t
