@@ -1,6 +1,6 @@
 (* Lightweight thread library for Objective Caml
  * http://www.ocsigen.org/lwt
- * Module Lwt_glib
+ * Interface Lwt_glib
  * Copyright (C) 2009 Jérémie Dimino
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,13 +23,24 @@
 (** Glib integration *)
 
 (** This modules is intended to allow the use of Lwt in GTK
-    applications. It integrates the Lwt scheduler into the glib main
-    loop so light-weight threadscan run while GTK is running. *)
+    applications.
 
-val init : ?setlocale:bool -> unit -> unit
-  (** Initialises glib and integrates the Lwt scheduler into the glib
-      main loop. *)
+    If you are using GTK and Lwt in the same application, then you
+    must call {!install} at the beginning of you program. *)
 
-val quit : unit -> unit
-  (** Calls [GMain.quit] and remove the Lwt scheduler from the glib
-      main loop *)
+val install : ?mode : [ `glib_into_lwt | `lwt_into_glib ] -> unit -> unit
+  (** Install the Glib<->Lwt integration.
+
+      If [mode] is [`glib_into_lwt] then glib will use the Lwt main
+      loop, and if [mode] is [`lwt_into_glib] then Lwt will use the
+      Glib main loop.
+
+      The first mode is better but for some unknown reason it does not
+      work under Windows, so the second is used as default on Windows
+      while the first one is used as default on Unix.
+
+      If the integration is already active, this function does
+      nothing. *)
+
+val remove : unit -> unit
+  (** Remove the Glib<->Lwt integration. *)
