@@ -374,10 +374,30 @@ let writable ch =
 let set_state ch st =
   ch.state <- st
 
+let clear_events ch =
+  Lwt_sequence.iter_node_l (fun node -> Lwt_sequence.remove node; Lwt_sequence.get node ()) ch.hooks_readable;
+  Lwt_sequence.iter_node_l (fun node -> Lwt_sequence.remove node; Lwt_sequence.get node ()) ch.hooks_writable;
+  begin
+    match ch.event_readable with
+      | Some ev ->
+          ch.event_readable <- None;
+          Lwt_engine.stop_event ev
+      | None ->
+          ()
+  end;
+  begin
+    match ch.event_writable with
+      | Some ev ->
+          ch.event_writable <- None;
+          Lwt_engine.stop_event ev
+      | None ->
+          ()
+  end
+
 let abort ch e =
   if ch.state <> Closed then begin
     set_state ch (Aborted e);
-    Lwt_engine.fake_io ch.fd
+    clear_events ch
   end
 
 let unix_file_descr ch = ch.fd
@@ -563,6 +583,7 @@ let openfile name flags perms =
 let close ch =
   if ch.state = Closed then check_descriptor ch;
   set_state ch Closed;
+  clear_events ch;
   return (Unix.close ch.fd)
 
 #else
@@ -574,6 +595,7 @@ external close_free : [ `unix_close ] job -> unit = "lwt_unix_close_free" "noall
 let close ch =
   if ch.state = Closed then check_descriptor ch;
   set_state ch Closed;
+  clear_events ch;
   execute_job (close_job ch.fd) close_result close_free
 
 #endif
@@ -686,6 +708,35 @@ let ftruncate ch offset =
 
 #endif
 
+(* +-----------------------------------------------------------------+
+   | Syncing                                                         |
+   +-----------------------------------------------------------------+ *)
+
+external fsync_job : Unix.file_descr -> [ `unix_fsync ] job = "lwt_unix_fsync_job"
+external fsync_result : [ `unix_fsync ] job -> unit = "lwt_unix_fsync_result"
+external fsync_free : [ `unix_fsync ] job -> unit = "lwt_unix_fsync_free"
+
+let fsync ch =
+  check_descriptor ch;
+  execute_job (fsync_job ch.fd) fsync_result fsync_free
+
+#if HAVE_FDATASYNC
+
+external fdatasync_job : Unix.file_descr -> [ `unix_fdatasync ] job = "lwt_unix_fdatasync_job"
+external fdatasync_result : [ `unix_fdatasync ] job -> unit = "lwt_unix_fdatasync_result"
+external fdatasync_free : [ `unix_fdatasync ] job -> unit = "lwt_unix_fdatasync_free"
+
+let fdatasync ch =
+  check_descriptor ch;
+  execute_job (fdatasync_job ch.fd) fdatasync_result fdatasync_free
+
+#else
+
+let fdatasync ch =
+  fail (Lwt_sys.Not_available "fdatasync")
+
+#endif
+
 (* +-----------------------------------------------------------------+
    | File status                                                     |
    +-----------------------------------------------------------------+ *)
@@ -2238,7 +2289,6 @@ let tcflow ch act =
 let notification_buffer = String.create 4
 
 external init_notification : unit -> Unix.file_descr = "lwt_unix_init_notification"
-external poke_notification : unit -> unit = "lwt_unix_poke_notification"
 external send_notification : int -> unit = "lwt_unix_send_notification_stub"
 external recv_notifications : unit -> int array = "lwt_unix_recv_notifications"
 
@@ -2251,28 +2301,11 @@ let handle_notification id =
     | None ->
         ()
 
-let pid = ref (Unix.getpid ())
-let ev_notifications = ref Lwt_engine.fake_event
-
 let rec handle_notifications ev =
   (* Process available notifications. *)
-  Array.iter handle_notification (recv_notifications ());
-  (* If the pid changed, create a new notification file descriptor. *)
-  let current_pid = Unix.getpid () in
-  if !pid <> current_pid then begin
-    pid := current_pid;
-    (* Stop monitoring the current notification file descriptor. *)
-    Lwt_engine.stop_event !ev_notifications;
-    (* Resend a notification to be sure other process using the same
-       file descriptor won't get stuck. *)
-    poke_notification ();
-    (* Reinitialise the notification system with a new file
-       descriptor. *)
-    ev_notifications := Lwt_engine.on_readable (init_notification ()) handle_notifications;
-  end
+  Array.iter handle_notification (recv_notifications ())
 
-let () =
-  ev_notifications := Lwt_engine.on_readable (init_notification ()) handle_notifications
+let event_notifications = ref (Lwt_engine.on_readable (init_notification ()) handle_notifications)
 
 (* +-----------------------------------------------------------------+
    | Signals                                                         |
@@ -2317,6 +2350,16 @@ let disable_signal_handler = Lazy.force
    | Processes                                                       |
    +-----------------------------------------------------------------+ *)
 
+let fork () =
+  match Unix.fork () with
+    | 0 ->
+        Lwt_engine.stop_event !event_notifications;
+        (* Reinitialise the notification system. *)
+        event_notifications := Lwt_engine.on_readable (init_notification ()) handle_notifications;
+        0
+    | pid ->
+        pid
+
 type process_status =
     Unix.process_status =
   | WEXITED of int
