@@ -230,6 +230,11 @@ val abort : file_descr -> exn -> unit
 
 (** {6 Process handling} *)
 
+val fork : unit -> int
+  (** [fork ()] does the same as [Unix.fork]. You must use this
+      function instead of [Unix.fork] when you want to use Lwt in the
+      child process. *)
+
 type process_status =
     Unix.process_status =
   | WEXITED of int
@@ -347,6 +352,18 @@ val truncate : string -> int -> unit Lwt.t
 val ftruncate : file_descr -> int -> unit Lwt.t
   (** Wrapper for [Unix.ftruncate] *)
 
+(** {6 Syncing} *)
+
+val fsync : file_descr -> unit Lwt.t
+  (** Synchronise all data and metadata of the file descriptor with
+      the disk. On Windows it uses [FlushFileBuffers]. *)
+
+val fdatasync : file_descr -> unit Lwt.t
+  (** Synchronise all data (but not metadata) of the file descriptor
+      with the disk.
+
+      Note that [fdatasync] is not available on all platforms. *)
+
 (** {6 File status} *)
 
 type file_kind =
